/******************************************************************************
* Copyright (c) 2018, Sylvain Corlay and Johan Mabille, Wolf Vollprecht and   *
* Martin Renou                                                                *
*                                                                             *
* Distributed under the terms of the BSD 3-Clause License.                    *
*                                                                             *
* The full license is in the file LICENSE, distributed with this software.    *
*******************************************************************************/

#ifndef XLEAFLET_GET_BASEMAPS_PATH_HPP
#define XLEAFLET_GET_BASEMAPS_PATH_HPP

#include <string>

namespace xlf
{
    std::string get_basemaps_path()
    {
        static std::string basemaps_path = "C:/bld/xleaflet_1562275428188/_h_env/Library/share/xleaflet/basemaps.json";
        return basemaps_path;
    }
}

#endif
