/******************************************************************************
* Copyright (c) 2018, Sylvain Corlay and Johan Mabille, and Wolf Vollprecht   *
*                                                                             *
* Distributed under the terms of the BSD 3-Clause License.                    *
*                                                                             *
* The full license is in the file LICENSE, distributed with this software.    *
*******************************************************************************/

#ifndef XLEAFLET_CONFIG_CLING_HPP
#define XLEAFLET_CONFIG_CLING_HPP

#pragma cling add_library_path("C:/bld/xleaflet_1562275428188/_h_env/Library/lib")
#pragma cling load("libxleaflet")

#endif
